"use strict";

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var fs = require("fs");

var path = require("path");

var marked = require("marked").marked;

var NjkEnv = require("./njk-env.js");

var Utils = require("./utils.js");

var PageParser = require("./page-parser.js");

var Logger = require("./logger.js");

var hl = require("highlight.js");

marked.setOptions({
  highlight: function highlight(code, lang) {
    return lang ? hl.highlight(code, {
      language: lang
    }).value : hl.highlightAuto(code).value;
  }
});

function generate(srcFolderPath, targetFolderPath, customFilters, customGlobals) {
  var logLevel = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : "info";
  var layoutsFolderPath = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : path.join(srcFolderPath, "assets", "layouts");
  var dataFolderPath = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : path.join(srcFolderPath, "assets", "data");
  var env = NjkEnv.getEnv(customFilters, customGlobals);
  var logger = new Logger(logLevel);
  var pageParser = PageParser(srcFolderPath, targetFolderPath, layoutsFolderPath, dataFolderPath, logger);
  pageParser.clear();
  pageParser.parseFiles();
  var pages = pageParser.getPages();

  var globalData = _objectSpread(_objectSpread({}, pageParser.getData()), {}, {
    pages: pages
  });

  var _iterator = _createForOfIteratorHelper(pages),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var page = _step.value;
      logger.debug("Processing markdown: ".concat(page.source.relativeFilePath));

      var data = _objectSpread(_objectSpread({}, globalData), page.data);

      if (page.data.repeat) {
        var iterations = pageParser.getRepeatIterations(page, env, data);

        var _iterator2 = _createForOfIteratorHelper(iterations),
            _step2;

        try {
          for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
            var iteration = _step2.value;

            var iterationData = _objectSpread(_objectSpread({}, data), iteration);

            renderPages(env, page, iterationData, pageParser);
          }
        } catch (err) {
          _iterator2.e(err);
        } finally {
          _iterator2.f();
        }
      } else {
        renderPages(env, page, data, pageParser);
      }
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
}

function renderPages(env, page, data, pageParser) {
  var paginations = pageParser.getPaginations(page, env, data);

  if (paginations && paginations.length) {
    var _iterator3 = _createForOfIteratorHelper(paginations),
        _step3;

    try {
      for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
        var pagination = _step3.value;

        var paginationData = _objectSpread(_objectSpread({}, data), pagination);

        renderPage(env, page, paginationData, pageParser);
      }
    } catch (err) {
      _iterator3.e(err);
    } finally {
      _iterator3.f();
    }
  } else {
    renderPage(env, page, data, pageParser);
  }
}

function renderPage(env, page, data, pageParser) {
  var content = page.content;
  data = pageParser.fillDefinedDataSources(page, env, data);
  var renderedContent = env.renderString(content, data);
  renderedContent = pageParser.renderWithLayout(env, data, page.data.layout, renderedContent);
  renderedContent = marked(renderedContent); // renderedContent = hl.highlightAuto(renderedContent).value;

  var folderPath = env.renderString(page.target.folderPath, data);
  var filePath = env.renderString(page.target.filePath, data);
  Utils.createDirectory(folderPath);
  fs.writeFileSync(filePath, renderedContent);
}

module.exports = {
  generate: generate
};