"use strict";

var nunjucks = require("nunjucks");

var nunjucksDate = require("nunjucks-date");

var slug = require("slug");

nunjucksDate.setDefaultFormat("MMMM Do YYYY, h:mm:ss a");

var getEnv = function getEnv(customFilters, customGlobals) {
  var env = nunjucks.configure({
    autoescape: true
  });
  env.addFilter("dateFormat", nunjucksDate);
  env.addFilter("filterPagesByType", function (pages, type) {
    return pages.filter(function (page) {
      return page.data.type === type;
    });
  });
  env.addFilter("sortPagesByDate", function (pages) {
    return pages.sort(function (pageA, pageB) {
      var dateA = null;
      var dateB = null;

      if (pageB.data.date) {
        dateB = new Date(pageB.data.date);
      }

      if (pageA.data.date) {
        dateA = new Date(pageA.data.date);
      }

      if (dateB) {
        if (dateA) {
          return dateB - dateA;
        }

        return 1;
      }

      if (dateA) {
        return -1;
      }

      return 0;
    });
  });
  env.addFilter("slug", function (value) {
    if (!value) return value;
    return slug(value);
  });
  env.addFilter("nth", function (array, n) {
    return array[n];
  });
  env.addFilter("filterBy", function (array, property, value) {
    var not = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
    value = this.env.renderString(value.toString(), this.ctx);
    return array.filter(function (item) {
      if (not) return item[property] !== value;
      return item[property] == value;
    });
  });
  env.addFilter("firstBy", function (array, property, value) {
    var not = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
    value = this.env.renderString(value.toString(), this.ctx);
    return array.filter(function (item) {
      if (not) return item[property] !== value;
      return item[property] == value;
    })[0];
  });

  for (var filterName in customFilters) {
    if (customFilters.hasOwnProperty(filterName)) {
      env.addFilter(filterName, customFilters[filterName]);
    }
  }

  for (var globalName in customGlobals) {
    if (customGlobals.hasOwnProperty(globalName)) {
      env.addGlobal(globalName, customGlobals[globalName]);
    }
  }

  return env;
};

exports.getEnv = getEnv;