"use strict";

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var fs = require("fs");

var path = require("path");

var matter = require("gray-matter");

var walk = require("walk");

var minify = require("@node-minify/core");

var cleanCSS = require("@node-minify/clean-css");

var Utils = require("./utils.js");

var he = require("he");

var sass = require("node-sass");

function PageParser(sourceFolderPath, targetFolderPath, layoutsFolderPath, dataFolderPath, logger) {
  var instance = {};
  instance.pages = [];
  instance.data = [];
  instance.layouts = [];
  instance.sourceFolderPath = sourceFolderPath;
  instance.targetFolderPath = targetFolderPath;
  instance.layoutsFolderPath = layoutsFolderPath;
  instance.dataFolderPath = dataFolderPath;
  instance.logger = logger;

  instance.clear = function () {
    instance.pages = [];
    Utils.clearDirectory(instance.targetFolderPath);
  };

  instance.getPages = function () {
    return instance.pages;
  };

  instance.getData = function () {
    return instance.data;
  };

  instance.renderWithLayout = function (env, data, name, childContent) {
    if (name && instance.layouts[name]) {
      var layout = instance.layouts[name];

      if (layout.data.layout) {
        var renderedContent = this.renderWithLayout(env, data, layout.data.layout, layout.content);

        if (renderedContent) {
          return env.renderString(renderedContent, _objectSpread(_objectSpread({}, data), {}, {
            content: "\n\n".concat(childContent, "\n")
          }));
        }
      }

      return env.renderString(layout.content, _objectSpread(_objectSpread({}, data), {}, {
        content: "\n\n".concat(childContent, "\n")
      }));
    }

    return childContent;
  };

  instance.parseFiles = function () {
    walk.walkSync(instance.sourceFolderPath, {
      listeners: {
        file: function file(folderPath, fileStats, next) {
          var fileInfo = Utils.getDestFileInfo(instance.targetFolderPath, instance.sourceFolderPath, folderPath, fileStats.name, instance.layoutsFolderPath, instance.dataFolderPath);
          var fileName = fileInfo.source.fileName;
          var filePath = fileInfo.source.filePath;

          try {
            switch (fileInfo.source.type) {
              case Utils.FileTypes.Layout:
                instance.logger.debug("Parsing layout: ".concat(fileName));
                var layoutContent = fs.readFileSync(filePath, "utf8");
                var layoutMatter = matter(layoutContent);
                instance.layouts[fileName] = {
                  content: layoutMatter.content,
                  data: layoutMatter.data
                };
                break;

              case Utils.FileTypes.Json:
                instance.logger.debug("Parsing json data: ".concat(fileName));
                instance.data[fileName] = JSON.parse(fs.readFileSync(filePath, "utf8"));
                break;

              case Utils.FileTypes.MarkDown:
              case Utils.FileTypes.Nunjucks:
                instance.logger.debug("Parsing markdown: ".concat(fileInfo.source.relativeFilePath));
                var pageContent = fs.readFileSync(filePath, "utf8");
                var pageMatter = matter(pageContent);
                var slug = pageMatter.data.slug;

                var _targetFolderPath = slug ? path.join(instance.targetFolderPath, slug) : fileInfo.target.relativeFolderPath;

                var targetFilePath = path.join(_targetFolderPath, fileInfo.target.fileName + fileInfo.target.fileExtension);
                fileInfo = _objectSpread(_objectSpread({}, fileInfo), {}, {
                  url: slug ? slug : fileInfo.url,
                  target: _objectSpread(_objectSpread({}, fileInfo.target), {}, {
                    relativeFolderPath: _targetFolderPath,
                    relativeFilePath: targetFilePath,
                    folderPath: path.resolve(_targetFolderPath),
                    filePath: path.resolve(targetFilePath)
                  })
                });
                instance.pages.push(_objectSpread(_objectSpread({}, fileInfo), {}, {
                  content: pageMatter.content,
                  data: pageMatter.data
                }));
                break;

              case Utils.FileTypes.Stylesheet:
                instance.logger.debug("Minifying stylesheet: ".concat(fileInfo.source.relativeFilePath));
                Utils.createDirectory(fileInfo.target.folderPath);
                minify({
                  compressor: cleanCSS,
                  input: fileInfo.source.filePath,
                  output: fileInfo.target.filePath,
                  callback: function callback(err, min) {}
                });
                break;

              case Utils.FileTypes.Scss:
                instance.logger.debug("Compiling Sass: ".concat(fileInfo.source.relativeFilePath));
                Utils.createDirectory(fileInfo.target.folderPath);
                var sassResult = sass.renderSync({
                  file: fileInfo.source.filePath,
                  outputStyle: "compressed",
                  sourceMap: false,
                  omitSourceMapUrl: true
                });
                fs.writeFileSync(fileInfo.target.filePath, sassResult.css);
                break;

              default:
                Utils.createDirectory(fileInfo.target.folderPath);
                Utils.copyFileToTarget(fileInfo, next);
                instance.logger.debug("Copied [".concat(fileInfo.source.relativeFilePath, "] to [").concat(fileInfo.target.relativeFilePath, "]"));
                break;
            }
          } catch (error) {
            instance.logger.error(error);
          } finally {
            next();
          }
        }
      }
    });
  };

  instance.getRepeatIterations = function (page, env, data) {
    try {
      var dict = page.data.repeat;
      var iterations = [{}];

      for (var key in dict) {
        if (dict.hasOwnProperty(key)) {
          var length = iterations.length;

          for (var iterationIndex = 0; iterationIndex < length; iterationIndex++) {
            var dataSourceExpression = dict[key];

            var _length = new Number(env.renderString("{{ ".concat(dataSourceExpression, " | length }}"), _objectSpread(_objectSpread({}, data), iterations[iterationIndex])));

            var iterationCopies = void 0;

            for (var valueIndex = 0; valueIndex < _length; valueIndex++) {
              if (valueIndex === 0) {
                iterationCopies = [].concat(iterations);
              }

              var iterationCopy = _objectSpread({}, iterationCopies[iterationIndex]);

              var currentValue = JSON.parse(he.decode(env.renderString("{{ ".concat(dataSourceExpression, " | nth(").concat(valueIndex, ") | dump }}"), _objectSpread(_objectSpread({}, data), iterationCopy))));

              if (valueIndex === 0) {
                iterations[iterationIndex][key] = currentValue;
              } else {
                iterationCopy[key] = currentValue;
                iterations.push(iterationCopy);
              }
            }
          }
        }
      }

      return iterations;
    } catch (ex) {
      instance.logger.error("Error", ex);
    }
  };

  instance.getPaginations = function (page, env, data) {
    try {
      var paginationsData = [];
      if (!page.data.pagination) return paginationsData;

      var _iterator = _createForOfIteratorHelper(page.data.pagination),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var pagination = _step.value;
          var dataSourceExpression = pagination.from;
          var dataSource = JSON.parse(he.decode(env.renderString("{{ ".concat(dataSourceExpression, " | dump }}"), data)));
          var pageSize = pagination.size || 10;
          var pages = Utils.paginate(dataSource, pageSize);
          var pageCount = pages.length;
          var pageNumber = 1;

          do {
            var dataSet = pages[pageNumber - 1];
            if (!dataSet) dataSet = [];
            var pagedData = JSON.parse("{\n              \"".concat(pagination.alias, "\": {\n                \"data\": ").concat(JSON.stringify(dataSet), ",\n                \"pageNumber\": ").concat(pageNumber, ",\n                \"pageSize\": ").concat(pageSize, ",\n                ").concat(pageNumber !== 1 ? "\"firstPageNumber\": 1," : "", "\n                ").concat(pageNumber > 1 ? "\"prevPageNumber\": ".concat(pageNumber - 1, ",") : "", "\n                ").concat(pageNumber < pageCount ? "\"nextPageNumber\": ".concat(pageNumber + 1, ",") : "", "\n                ").concat(pageNumber !== pageCount ? "\"lastPageNumber\": ".concat(pageCount, ",") : "", "\n                \"pages\": ").concat(pageCount, "\n              }\n            }"));
            paginationsData.push(pagedData);
            pageNumber++;
          } while ((pageNumber - 1) * pageSize < dataSource.length);
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      return paginationsData;
    } catch (ex) {
      instance.logger.error("Error", ex);
    }
  };

  instance.fillDefinedDataSources = function (page, env, data) {
    try {
      if (page.data.define) {
        var dict = page.data.define;

        for (var key in dict) {
          if (dict.hasOwnProperty(key)) {
            data[key] = JSON.parse(he.decode(env.renderString("{{ ".concat(dict[key], "  | dump }}"), data)));
          }
        }
      }
    } catch (ex) {
      instance.logger.error("Error", ex);
    }

    return data;
  };

  return instance;
}

module.exports = PageParser;