"use strict";

var fs = require("fs");

var path = require("path");

var rimraf = require("rimraf");

String.prototype.removeStart = function (start) {
  var out = this;
  if (0 === out.indexOf(start)) out = out.slice(start.length);
  return out;
};

String.prototype.removeEnd = function (end) {
  var out = this;
  var length = out.length - end.length;
  if (out.indexOf(end) === length) out = out.substring(0, length);
  return out;
};

var FileTypes = {
  MarkDown: "MarkDown",
  Stylesheet: "Stylesheet",
  Scss: "Stylesheet-SCSS",
  JavaScript: "JavaScript",
  HTML: "HTML",
  Binary: "Binary",
  Layout: "Layout",
  Nunjucks: "Nunjucks",
  Json: "Json",
  None: "None"
};

var getDestFileInfo = function getDestFileInfo(mainTargetFolder, mainSourceFolder, sourceFolderPath, sourceFileName, layoutsFolderPath, dataFolderPath) {
  var sourceFilePath = path.join(sourceFolderPath, sourceFileName);
  var sourceFileExtension = path.extname(sourceFileName);
  var sourceFileBaseName = path.basename(sourceFileName, sourceFileExtension);
  var sourceFileType = FileTypes.None;
  var targetFileType = null;
  var targetFolderPath = path.join(mainTargetFolder, sourceFilePath.substr(mainSourceFolder.length, sourceFilePath.length - mainSourceFolder.length - sourceFileName.length));
  var targetFileBaseName = sourceFileBaseName;
  var targetExtension = sourceFileExtension;
  var targetFilePath = sourceFilePath;

  switch (sourceFileExtension) {
    case ".md":
      sourceFileType = FileTypes.MarkDown;
      targetExtension = ".html";
      targetFileType = FileTypes.HTML;

      if (sourceFileBaseName !== "index") {
        targetFolderPath = path.join(targetFolderPath, sourceFileBaseName);
        targetFileBaseName = "index";
      }

      break;

    case ".njk":
      sourceFileType = FileTypes.Nunjucks;
      targetExtension = ".html";
      targetFileType = FileTypes.HTML;

      if (sourceFileBaseName !== "index") {
        targetFolderPath = path.join(targetFolderPath, sourceFileBaseName);
        targetFileBaseName = "index";
      }

      break;

    case ".css":
      sourceFileType = FileTypes.Stylesheet;
      break;

    case ".scss":
    case ".sass":
      sourceFileType = FileTypes.Scss;
      targetFileType = FileTypes.Stylesheet;
      targetExtension = ".css";
      break;

    case ".js":
      sourceFileType = FileTypes.JavaScript;
      break;

    case ".html":
      if (sourceFolderPath.startsWith(layoutsFolderPath)) sourceFileType = FileTypes.Layout;else sourceFileType = FileTypes.HTML;
      break;

    case ".json":
      if (sourceFolderPath.startsWith(dataFolderPath)) sourceFileType = FileTypes.Json;else sourceFileType = FileTypes.None;
      break;

    case ".jpg":
    case ".jpeg":
    case ".png":
    case ".ico":
      sourceFileType = FileTypes.Binary;
      break;

    default:
      sourceFileType = FileTypes.None;
      break;
  }

  targetFilePath = path.join(targetFolderPath, targetFileBaseName + targetExtension);
  var url = targetFilePath.removeStart(mainTargetFolder).removeEnd("index.html");
  return {
    url: url,
    target: {
      relativeFolderPath: targetFolderPath,
      relativeFilePath: targetFilePath,
      folderPath: path.resolve(targetFolderPath),
      filePath: path.resolve(targetFilePath),
      fileName: targetFileBaseName,
      fileExtension: targetExtension,
      type: targetFileType ? targetFileType : sourceFileType
    },
    source: {
      relativeFolderPath: sourceFolderPath,
      relativeFilePath: sourceFilePath,
      folderPath: path.resolve(sourceFolderPath),
      filePath: path.resolve(sourceFilePath),
      fileName: sourceFileBaseName,
      fileExtension: sourceFileExtension,
      type: sourceFileType
    }
  };
};

var copyFileToTarget = function copyFileToTarget(fileInfo, callback) {
  fs.copyFileSync(fileInfo.source.filePath, fileInfo.target.filePath);
  if (callback) callback();
};

var createDirectory = function createDirectory(folderPath) {
  if (!fs.existsSync(folderPath)) {
    fs.mkdirSync(folderPath, {
      recursive: true
    });
  }
};

var clearDirectory = function clearDirectory(folderPath) {
  rimraf.sync(folderPath);
};

var paginate = function paginate(arr, size) {
  return arr.reduce(function (acc, val, i) {
    var idx = Math.floor(i / size);
    var page = acc[idx] || (acc[idx] = []);
    page.push(val);
    return acc;
  }, []);
};

exports.FileTypes = FileTypes;
exports.getDestFileInfo = getDestFileInfo;
exports.copyFileToTarget = copyFileToTarget;
exports.createDirectory = createDirectory;
exports.clearDirectory = clearDirectory;
exports.paginate = paginate;